{ Tool for timing evaluation of Delphi Pascal code fragments.
  Timing is based on the cpu clock , resolution  ca 10ns (=10^-8 s).
  May , version 2.20 published as freeware by author A. Baars
  , any comments :  antonie.baars@wxs.nl              }

{
  This is one of the best freeware tools ever created, but it became a little
  bit dated by 2002, so I preserved the original code (2.2 1998) just formatted
  it and replaced the original CPU speed calibrator.

  To keep version consistency from original I called it Ver 2.3
  Igor P. Zenkov <igor@zenkov.com>, Jan 10, 2002
  You can download it from http://www.zenkov.com
}

unit Z_prof;

interface

uses
  Windows,  // DWord
  Classes,  //
  Controls, //
  Forms,    //
  Grids,    //
  Buttons,  //
  ExtCtrls, //
  StdCtrls; //

const
  NMarks = 20;

type
  MarkRecord = record
    Passes: DWord;
    t: Int64;
    Ctime, TotalTime, Fastest, PrevTime: Extended;
    Active: boolean;
  end;

  TProfiler = class(TForm)
    Panel1: TPanel;
    ClearBtn: TSpeedButton;
    DrawGrid1: TDrawGrid;
    RadioGroup1: TRadioGroup;
    CpuLbl: TLabel;
    CalibrateBtn: TSpeedButton;
    AboutBtn: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure CalibrateBtnClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
  private
    RMark: array [1..NMarks] of MarkRecord;
    procedure GetTime(nr: integer);
    function GetTimeDiff(nr: integer): Extended;
    procedure CorrectForOverhead;
    procedure CalibrateCPU;
    procedure Tablecontent(Sender: TObject; Col, Row: Longint; Rect: TRect; State: TGridDrawState);
  public
    fdVisible, frVisible: boolean;
    procedure Mark(nr: integer; aActive: boolean);
  end;

  TZprofiler = class(TComponent)
  private
    Profiler: TProfiler;
    procedure SetfdVisible(b1: boolean);
    function GetfdVisible: boolean;
    procedure SetfrVisible(b1: boolean);
    function GetfrVisible: boolean;
  public
    constructor Create(aowner:tcomponent);override;
    destructor Destroy;override;
    procedure Mark(nr: integer; aActive: boolean);
    property Visible: boolean read GetfrVisible write SetfrVisible;
  published
    property VisibleInDesignMode: boolean read GetfdVisible write SetfdVisible;
  end;

procedure Register;
var
  Profile: TProfiler;

implementation

{$R *.DFM}

uses
  SysUtils; // IntToStr

type
  TCompRec = record
    case Integer of
      0: (Lo, Hi: DWord);
      1: (I64: Int64);
  end;

const
  InstalledCount: integer = 0;

var
  gGridHandle: HWnd;
  m, n: TCompRec;
  gLastMark: Integer;
  gCpuClock, gResidualTime: Extended;

procedure Register;
begin
  RegisterComponents('Samples', [TZProfiler]);
end;

constructor TZProfiler.Create(aOwner: TComponent);
begin
  inherited create(aowner);
  if InstalledCount = 0 then
    Profile := TProfiler.Create(Application.MainForm);
  Profile.Visible := True;
  Inc(InstalledCount);
  Profiler := Profile;
  if (csdesigning in ComponentState) then
  begin
    if VisibleInDesignMode then
      Profiler.Show else Profiler.Hide;
  end;
end;

destructor TZProfiler.Destroy;
begin
  Dec(InstalledCount);
  if InstalledCount = 0 then
    Profile.Free;
  inherited Destroy;
end;

procedure TZProfiler.SetfdVisible(b1: boolean);
begin
  Profile.fdVisible := b1;
  if ((csDesigning in ComponentState) and not b1) then
    Profiler.Hide
  else
    Profiler.Show;
end;

function TZProfiler.GetfdVisible: boolean;
begin
  Result := Profile.fdVisible;
end;

procedure TZProfiler.SetfrVisible(b1: boolean);
begin
  Profile.frVisible := b1;
  if (csDesigning in ComponentState) then Exit;
  if b1 then
    Profiler.Show
  else
    Profiler.Hide;
end;

function TZProfiler.GetfrVisible: boolean;
begin
  Result := Profile.frVisible;
end;


procedure TZProfiler.Mark(nr: integer; aActive: boolean);
begin
  Profiler.Mark(nr, aActive);
end;

procedure TProfiler.TableContent(Sender: TObject; Col, Row: Longint; Rect: TRect;
 State: TGridDrawState);
var
  mtext, mmtext: string;
  div1: Extended;
begin
  mtext := ''; mmtext := ''; div1 := 1;

  case RadioGroup1.ItemIndex of
    0: begin mmtext := 's';  div1 := 1e-6; end;
    1: begin mmtext := 'ms'; div1 := 1e-3; end;
    2: begin mmtext := 's'; div1 := 1;    end;
    3: begin mmtext := 'ns'; div1 := 1000; end; // Added nanoseconds, Igor Zenkov
  end;

  if Row = 0 then
    case Col of
      0: mtext := 'marknr';
      1: mtext := 'passes';
      2: mtext := 'time /' + mmtext;
      3: mtext := 'last /' + mmtext;
      4: mtext := 'mean /' + mmtext;
      5: mtext := 'fastest /' + mmtext;
    end
  else
    with RMark[Row] do
      case Col of
        0: mtext := IntToStr(Row);
        1: mtext := IntToStr(Passes);
        2: mtext := FloatToStrF((div1 * TotalTime)/gCpuClock, ffFixed, 8, 2);
        3: mtext := FloatToStrF((div1 * CTime)/gCpuClock, ffFixed, 8, 2);
        4: if Passes > 0 then mtext := FloatToStrF((div1 * TotalTime)/(gCpuClock * Passes), ffFixed, 8, 2);
        5: if Passes > 0 then mtext := FloatToStrF((div1 * Fastest)/gCpuClock, ffFixed, 8, 2);
      end;

  with Rect, TDrawGrid(Sender).Canvas do
    TextRect(Rect, Left + (Right - Left - TextWidth(mtext)) div 2, Top +
    (Bottom - Top - TextHeight(mtext)) div 2, mtext);
end;

procedure MMTime; assembler;
asm
  DB 0FH
  DB 031H
  mov TCompRec(m).hi, edx
  mov TCompRec(m).lo, eax
end;

procedure NNTime; assembler;
asm
  DB 0FH
  DB 031H
  mov TCompRec(n).hi, edx
  mov TCompRec(n).lo, eax
end;

procedure TProfiler.GetTime(nr: integer);
begin
  RMark[nr].t := m.I64;
end;

procedure TProfiler.CorrectForOverhead;
var
  i: integer;
  OverheadTime: Extended;
begin
  OverheadTime := n.I64 - RMark[gLastMark].t;
  for i := 1 to NMarks do
    with RMark[i] do
      if Active then TotalTime := TotalTime - OverheadTime - gResidualTime;
end;

function TProfiler.GetTimeDiff(nr: integer): Extended;
begin
  Result := m.I64 - RMark[nr].t;
  Self.GetTime(nr);
end;

procedure TProfiler.Mark(nr: integer; aActive: boolean);
begin
  if (RMark[nr].Active = aActive) then Exit;

  MMTime;
  CorrectForOverhead;
  if aActive then
    //begin
      //if (not RMark[nr].Active) then <- redundant Igor Zenkov
      begin
        Self.GetTime(nr);
        RMark[nr].Active := True;
      end{;}
    //end
  else
    //if RMark[nr].Active then  <- redundant Igor Zenkov
      with RMark[nr] do
      begin
        TotalTime := TotalTime + Self.GetTimeDiff(nr);
        CTime     := TotalTime - PrevTime;
        PrevTime  := TotalTime;
        Active    := False;
        Inc(Passes);
        if (Passes = 1) then
          Fastest := CTime
        else
          if CTime < Fastest then
            Fastest := CTime;
        InvalidateRect(gGridHandle, nil, True);
      end;

  gLastMark := nr; // keep a copy for the overhead time subtraction
  NNTime;
end;

procedure TProfiler.FormCreate(Sender: TObject);
begin
  DrawGrid1.OnDrawCell := TableContent;
  drawGrid1.ColWidths[0] := 50;
  drawGrid1.ColWidths[1] := 50;
  gGridHandle := DrawGrid1.Handle;
  CalibrateBtnClick(Self);
end;

// Added by Igor Zenkov, this code has a lot better accuracy
// test time: 1/2 of second
function GetCpuClockSpeed: Extended;
var
  vFreq, t1, t2, vDelta: Int64;
begin
  QueryPerformanceFrequency(vFreq); // get frequency
  vDelta := vFreq div 2;
  QueryPerformanceCounter(t1);
  repeat                            // wait for tickchange
    QueryPerformanceCounter(t2);
  until t1 < t2;
  MMTime;
  repeat                            // wait for tickchange
    QueryPerformanceCounter(t2);
  until (t2 - t1) > vDelta;
  NNTime;
  Result := 2e-6 * (n.I64 - m.I64); // MHz
end;

// Modified by Igor Zenkov
procedure TProfiler.CalibrateCPU;
begin
  gCpuClock := GetCpuClockSpeed; // MHz
  CpuLbl.Caption := Format('clock: %f MHz', [gCpuClock]);
end;

// Commented out by Igor Zenkov
{procedure TProfiler.CalibrateCPU;
var
  t1: DWord;
begin
  t1 := GetTickCount;
  while t1 = GetTickCount do; // wait for tickchange
  MMTime;
  while GetTickCount < (t1 + 400) do;
  NNTime;
  gCpuClock := 2.5e-6 * (n - m); // MHz
  CpuLbl.Caption := Format('clock: %f MHz', [gCpuClock]);
end;}

procedure TProfiler.ClearBtnClick(Sender: TObject);
var
  i: integer;
begin
  for i := 1 to NMarks do
    with RMark[i] do
    begin
      Passes := 0; CTime := 0; TotalTime := 0; Fastest := 0; PrevTime := 0;
      Active := False;
    end;
  gLastMark := 1;
  InvalidateRect(gGridHandle, nil, True);
end;

procedure TProfiler.RadioGroup1Click(Sender: TObject);
begin
  InvalidateRect(gGridHandle, nil, True);
end;

procedure TProfiler.CalibrateBtnClick(Sender: TObject);
var
  i: integer;
begin
  CalibrateCPU;
  ClearBtnClick(Self);
  gResidualTime := 0;
  for i := 1 to 1000 do
  begin
    Mark(1, True); Mark(1, False);
  end;
  gResidualTime := RMark[1].TotalTime/1000;
  ClearBtnClick(Self);
end;

procedure TProfiler.AboutBtnClick(Sender: TObject);
const
  // Modified by Igor Zenkov
  //cTxt = 'Zprofiler 2.2: freeware program timing tool by A.Baars (1998)';
  cTxt = 'Zprofiler 2.3'#13 +
         'Freeware Program Timing Tool.'#13#13 +
         'ver 1.0 - Antonie Baars'#13 +
         'ver 2.2 - Antonie Baars, May 1998'#13 +
         'ver 2.3 - Igor Zenkov, Jan 2002';
begin
  MessageBox(Self.Handle, PChar(cTxt), 'About', MB_OK or MB_ICONINFORMATION);
end;

end.

